import numpy as np 
from skimage.measure import label 

def get_image_position_per_slice(liste_instances):
    liste = []
    for i in range(len(liste_instances)):
        liste.append(liste_instances[i].get_image_position())
    return liste 


def find_corresponding_z_spatial(liste_position, z_matrix):
    z_spatial = liste_position[z_matrix][2]
    return z_spatial


def get_number_of_roi(mask):
    if len(mask.shape) == 3 : 
        if np.max(mask) == 1 : 
            return 1 
        else : return int(np.max(mask))

    elif len(mask.shape) == 4 : 
        if np.max(mask) == 1 : 
            return mask.shape[3]
        else : return int(np.max(mask))


def get_list_SOPInstance_UID(liste_instances):
    liste = []
    for instance in liste_instances : 
        liste.append(instance.get_SOPInstanceUID())

    return liste 


def clean_mask(mask):
    """[summary]

    Args:
        mask ([type]): [binary mask [x,y,z]]
    """
    #Check if binary mask 
    
    if int(np.max(mask)) != 1 :
        raise Exception("Not a binary mask")

    empty_mask = np.zeros((mask.shape))
    for s in range(mask.shape[2]) : 
        slice = mask[:,:,s]
        if int(np.max(slice)) == 0 : 
            empty_mask[:,:,s] = slice
        else : 
            lw, num = label(slice, connectivity=2, return_num=True) #lw = 2D slice 
            item = np.arange(1, num+1).tolist()
            area = []
            for it in item : 
                area.append(len(np.where(lw== it)[0]))
            for ar in area : 
                feature = area.index(ar) + 1 
                if int(ar) < 3 : 
                    x,y = np.where(lw == feature)
                    lw[x,y] = 0 
            empty_mask[:,:,s] = lw 

    #binarize image 
    matrix = np.where(empty_mask>0, 1, 0)
    return matrix








    


import numpy as np
import SimpleITK as sitk 
import matplotlib.pyplot as plt
from scipy import ndimage
from skimage.feature import peak_local_max
from skimage import segmentation
import scipy
from skimage import morphology
from library_dicom.post_processing.PostProcess_Reader import PostProcess_Reader
from library_dicom.dicom_processor.tools.create_mip import *

class WatershedModel(PostProcess_Reader):

    def __init__(self, mask_path, pet_path, type):
        super().__init__(mask_path, pet_path, type)


    def get_labels_for_model(self, labelled_threshold_array, num_labels):
        label = []
        vol_roi = []
        for key in range(1, num_labels + 1) :
            volume_voxel = self.pet_spacing[0] * self.pet_spacing[1] * self.pet_spacing[2] * 10**(-3)
            number_pixel = len(np.where(labelled_threshold_array == key)[0])
            volume_roi = volume_voxel * number_pixel

            if volume_roi > float(30) :
                label.append(key)
                vol_roi.append(volume_roi)
        return label, vol_roi

    def get_pet_view(self, pet_array, labelled_mask):
        new_mask = np.zeros(self.size_matrix)
        new_mask[np.where(labelled_mask != 0)] = pet_array[np.where(labelled_mask !=0)]
        return new_mask.astype(np.uint8)


    def get_suv_values_matrix(self, label, labelled_threshold_mask):
        #x,y,z = self.size_matrix
        new_matrix = np.zeros(self.size_matrix)
        new_matrix[np.where(labelled_threshold_mask == label)] = self.pet_array[np.where(labelled_threshold_mask== label)]
        return new_matrix

    def get_mask_roi_matrix(self, label, labelled_threshold_mask):
        #x,y,z = self.size_matrix
        new_matrix = np.zeros(self.size_matrix)
        new_matrix[np.where(labelled_threshold_mask == label)] = 1
        return new_matrix.astype(np.uint8)



    def get_distance_map(self, threshold_matrix):
        return ndimage.distance_transform_edt(threshold_matrix)

    #def get_local_peak(self, distance_map, number_max_peak):
        #min_dist = int(2/np.mean(self.pet_spacing))
        #if min_dist == 0 : 
        #    return peak_local_max(distance_map, indices = False)
        #else : 

        #return peak_local_max(distance_map, indices = False, num_peaks= number_max_peak)

    def get_local_peak(self, distance_map):
        spacing = []
        spacing.append(float(self.pet_spacing[0]) * 10**(-1))
        spacing.append(float(self.pet_spacing[1]) * 10**(-1))
        spacing.append(float(self.pet_spacing[2]) * 10**(-1))
        
        min_dist = int(2/np.mean(spacing))
        #print("min dist :", min_dist)

        #localMax_1 = peak_local_max(distance_map, indices = True, min_distance= min_dist)
        #suv_max = []
        #for point in localMax_1 : 
        #    suv_max.append(self.pet_array[point[0], point[1], point[2]])

        #moy = np.mean(suv_max)
        return peak_local_max(distance_map, indices = False, min_distance=min_dist)


    def define_marker_array(self, localMax):
        #marker_array = np.zeros(self.size_matrix)
        #for marker in range(len(localMax)) : 
            #marker_array[localMax[marker][0], localMax[marker][1], localMax[marker][2]] = marker + 1

        marker_array, num_features = ndimage.label(localMax) #, structure=np.ones((3,3,3)))
        return marker_array.astype(np.uint8), num_features
 

    def watershed_segmentation(self, distance_map, marker_array, mask) : 
        labels = segmentation.watershed(distance_map, marker_array, mask = mask)
        return labels.astype(np.uint8)


    def watershed_model(self, threshold) : 
        binary_threshold_mask_img = self.get_binary_threshold_mask_img(threshold)
        binary_threshold_mask_array = self.remove_small_roi(binary_threshold_mask_img)
        labelled_threshold_array, num_labels = self.get_labelled_threshold_mask_array(binary_threshold_mask_array)
       

        label_coordonate = self.label_coordonate(labelled_threshold_array, num_labels)

        labels_for_model, vol_roi = self.get_labels_for_model(labelled_threshold_array, num_labels)
        for label in labels_for_model : 

            suv_values= self.get_suv_values_matrix(label, labelled_threshold_array)
            
            #distance_map = self.get_distance_map(suv_values)

            #localMax = self.get_local_peak(distance_map, number_max_peak)
            localMax = self.get_local_peak(suv_values)
            #local_min = morphology.local_minima(suv_values, allow_borders=False)

            marker_array, num_features = self.define_marker_array(localMax)

            if num_features != 0 : 
                #new_distance_map = -1 * distance_map
                new_label_mask = self.watershed_segmentation(-suv_values, marker_array, suv_values)
                new_coordonate = []
                for new_label in range(1, num_features + 1):
                    if len(np.where(new_label_mask == new_label)[0]) != 0 : 
                        new_coordonate.append(np.where(new_label_mask == new_label))

                label_coordonate[label] = new_coordonate


        liste_coordonate = self.extract_coordonate(label_coordonate)
        number_total_of_label = len(liste_coordonate)
        liste_label = np.arange(1, number_total_of_label + 1, 1)
        return self.watershed_matrix(liste_coordonate, liste_label), len(liste_label)


    def extract_coordonate(self, results):
        liste = []
        for key in range(1, len(results) + 1) : 
            number_coord = len(results[key])
            for i in range(number_coord) : 
                liste.append(results[key][i])
        return liste 

    def watershed_matrix(self, liste_coordonate, liste_label) :
        ws_matrix = np.zeros(self.size_matrix)
        for coordonate, label in zip(liste_coordonate, liste_label) :
            ws_matrix[coordonate] = label

        return ws_matrix.astype(np.uint8)


    def get_watershed_img(self, ws_array):
        img = sitk.GetImageFromArray(ws_array.transpose())
        img.SetSpacing(self.pet_spacing)
        img.SetDirection(self.pet_direction)
        img.SetOrigin(self.pet_origin)

        return img



import pydicom 
import numpy as np
import cv2 
from random import randrange
from library_dicom.dicom_processor.model.reader.Instance_RTSS import Instance_RTSS
from library_dicom.export_segmentation.tools.rtss_writer_tools import *

class ROIContourSequence : 

    def __init__(self, mask):
        self.mask = mask
        self.number_of_roi = get_number_of_roi(self.mask)


    def __get_contour_ROI(self, number_roi):

        results = {}
        slice = []
        if self.mask.shape[-1] != 1 : 
            binary_mask = np.array(self.mask[:,:,:,number_roi - 1], dtype=np.uint8)
        else : 
            binary_mask = np.zeros((self.mask.shape[0], self.mask.shape[1], self.mask.shape[2]))
            x,y,z = np.where(self.mask[:,:,:,0] == number_roi)
            binary_mask[x,y,z] = 1
            binary_mask = np.array(binary_mask, dtype=np.uint8 )

        for s in range(self.mask.shape[2]):
            contours, _ = cv2.findContours(binary_mask[:,:, s], cv2.RETR_TREE,cv2.CHAIN_APPROX_NONE) 
            if (contours != []) : 
                results[s] = contours
                slice.append(s)

        return results, slice 


    def pixel_to_spatial(self, number_roi, image_position, pixel_spacing, list_all_SOPInstanceUID, liste_instances):
        liste_position = get_image_position_per_slice(liste_instances)


        list_contours = []
        list_SOPInstanceUID = []
        x0,y0,z0 = image_position
        dx,dy,dz = pixel_spacing
        dict_contours, list_slice = self.__get_contour_ROI(number_roi)

        number_contour = len(dict_contours)

        for i in range(number_contour): #plusieurs contours dans une même slice 
            number_of_contour_in_slice = len(dict_contours[list_slice[i]])
            for j in range(number_of_contour_in_slice):
                number_point_contour = len(dict_contours[list_slice[i]][j])

                liste = []

                for point in range(number_point_contour): #[x,y]
                    x = dict_contours[list_slice[i]][j][point][0][0]
                    liste.append(x0 + x*dx) #+dx/2
                    y = dict_contours[list_slice[i]][j][point][0][1]
                    liste.append( y0 + y*dy) #+dy/2
                    z = list_slice[i] 
                    z_spatial = find_corresponding_z_spatial(liste_position, z)
                    #liste.append(z0 + z*dz )
                    liste.append(z_spatial) 


                list_SOPInstanceUID.append(list_all_SOPInstanceUID[z])
                list_contours.append(liste)


        return list_contours, list_SOPInstanceUID


    def __create_ContourImageSequence(self, ReferencedSOPClassUID, ReferencedSOPInstanceUID):
        ContourImageSequence = pydicom.sequence.Sequence()
        dataset = pydicom.dataset.Dataset()
        dataset.ReferencedSOPClassUID = ReferencedSOPClassUID
        dataset.ReferencedSOPInstanceUID = ReferencedSOPInstanceUID
        ContourImageSequence.append(dataset)
        return ContourImageSequence 

    def __create_ContourSequence(self, ReferencedSOPClassUID, list_ReferencedSOPInstanceUID, list_ContourData):
        ContourSequence = pydicom.sequence.Sequence()

        for ContourData,SOPInstanceUID in zip(list_ContourData,list_ReferencedSOPInstanceUID):
            dataset = pydicom.dataset.Dataset()
            dataset.ContourData = ContourData             
            dataset.ContourImageSequence = self.__create_ContourImageSequence(ReferencedSOPClassUID, SOPInstanceUID)             
            dataset.NumberOfContourPoints = len(ContourData)/3 
            dataset.ContourGeometricType = 'CLOSED_PLANAR'

            ContourSequence.append(dataset)
        return ContourSequence 


    @classmethod
    def get_random_colour(cls):
        max = 256
        return [randrange(max), randrange(max), randrange(max)]


    def create_ROIContourSequence(self, ReferencedSOPClassUID, image_position, pixel_spacing, list_all_SOPInstanceUID, liste_instances):
        ROIContourSequence = pydicom.sequence.Sequence()
        for number_roi in range(1, self.number_of_roi +1) : 
            dataset = pydicom.dataset.Dataset()
            dataset.ROIDisplayColor = self.get_random_colour()
            dataset.ReferencedROINumber = number_roi
            list_contour_data , list_SOP_instance_uid = self.pixel_to_spatial(number_roi, image_position, pixel_spacing, list_all_SOPInstanceUID, liste_instances)
            dataset.ContourSequence = self.__create_ContourSequence(ReferencedSOPClassUID, list_SOP_instance_uid, list_contour_data)
            ROIContourSequence.append(dataset)
        return ROIContourSequence 





import numpy as np 
import SimpleITK as sitk 


def read_img(img_path):
    img = sitk.ReadImage(img_path)
    array = sitk.GetArrayFromImage(img).transpose()
    return array , img.GetSpacing()


def normalize_pet(array):
    x,y,z = np.where(array != 0)
    suv = []
    for j in range(len(x)):
        suv.append(array[x[j], y[j], z[j]])

    suv_max = np.max(suv)

    for i in range(len(x)):
        array[x[i], y[i], z[i]] = (array[x[i], y[i], z[i]] * 255) / suv_max

    return array 

def read_inference(inference_path):
    return sitk.GetArrayFromImage(sitk.ReadImage(inference_path)).transpose()

    
def binarize_mask(mask_array) : 
    size = mask_array.shape 
    binarize = np.zeros(size)
    x,y,z = np.where(mask_array >= 0.5)
    binarize[x,y,z] = 1
    return binarize


import pydicom 
import numpy as np
from library_dicom.export_segmentation.tools.rtss_writer_tools import *

class RTROIObservationsSequence :

    def __init__(self, mask, results):
        self.mask = mask
        self.number_of_roi = get_number_of_roi(self.mask)
        self.results = results


    def create_RTROIObservationsSequence(self):
        RTROIObservationsSequence = pydicom.sequence.Sequence()
        for number_roi in range(1, self.number_of_roi +1) : 
            dataset = pydicom.dataset.Dataset()
            dataset.ReferencedROINumber = number_roi
            dataset.ObservationNumber = number_roi 

            dataset.RTROIInterpretedType = self.results["segmentAttributes"][0][number_roi-1]['RTROIInterpretedType']
            dataset.ROIInterpreter = ''
            
            RTROIObservationsSequence.append(dataset)

        return RTROIObservationsSequence 


import numpy as np
import SimpleITK as sitk 
import matplotlib.pyplot as plt
import scipy as sc
from sklearn import mixture
from skimage.measure import label
from library_dicom.post_processing.Mask3D import Mask3D
from library_dicom.post_processing.Mask4D import Mask4D


class PostProcess_Reader : 

    def __init__(self, mask_path, pet_path, type): #pt_path
        self.mask_path = mask_path
        self.pet_path = pet_path
        self.type = type

        self.pet_img = self.read_pet_img()
        self.pet_array = self.get_pet_array()
        self.size_matrix = self.pet_array.shape

        self.mask_object = self.get_mask_object()
        


    def get_mask_object(self):
        if self.type == '3d' :
            return Mask3D(self.mask_path)

        elif self.type == '4d' : 
            return Mask4D(self.mask_path)

        else : 
            raise Exception ('Not a 3D or 4D mask ')
        

    def get_mask_array(self):
        """
        return mask array from image with no threshold, 3D or 4D
        """
        return self.mask_object.read_mask_array()

    def read_pet_img(self):
        pt_img = sitk.ReadImage(self.pet_path)
        self.pet_origin = pt_img.GetOrigin()
        self.pet_direction = pt_img.GetDirection()
        self.pet_spacing = pt_img.GetSpacing()
        self.pet_size = pt_img.GetSize()
        return pt_img

    def get_pet_array(self): 
        return sitk.GetArrayFromImage(self.pet_img).transpose()

    #binary img/array with threshold
    def get_binary_threshold_mask_img(self, threshold):
        binary_array = self.get_binary_threshold_mask_array(threshold)
        new_mask_img = sitk.GetImageFromArray(binary_array.transpose())
        new_mask_img.SetOrigin(self.pet_origin)
        new_mask_img.SetSpacing(self.pet_spacing)
        new_mask_img.SetDirection(self.pet_direction)
        return new_mask_img 

        
    def get_binary_threshold_mask_array(self, threshold):
        """
        Return a 3d binary threshold mask 
        """
        if self.type == '4d' : 
            return self.mask_object.get_binary_threshold_mask(self.pet_array, threshold)
        elif self.type == '3d' : 
            return self.mask_object.get_binary_threshold_mask(threshold)
        else : 
            raise Exception ('not a 3d or 4d mask')


    #get informations
    def get_mask_img_spacing(self):
        return self.mask_object.get_mask_spacing()

    def get_mask_img_size(self):
        return self.mask_object.get_mask_size()
        
    def get_mask_img_origin(self):
        return self.mask_object.get_mask_origin()

    def get_mask_img_direction(self):
        return self.mask_object.get_mask_direction()
        
    #check
    def is_details_same(self):
        spacing = self.get_mask_img_spacing()
        size = self.get_mask_img_size()
        origin = self.get_mask_img_origin()
        direction = self.get_mask_img_direction()
        if (self.pet_spacing != spacing) or (self.pet_origin != origin) or (self.pet_direction != direction) or (self.pet_size != size) : 
            raise Exception ('Not the same spacing, size, origin or direction between mask and pet')

        else : return True 


    #remove small ROI
    def remove_small_roi(self, binary_img):
        if len(binary_img.GetSize()) != 3 : 
            raise Exception("Not a 3D mask, need to transform into 3D binary mask")
        
        else : 
            labelled_img = sitk.ConnectedComponent(binary_img)
            stats = sitk.LabelIntensityStatisticsImageFilter()
            stats.Execute(labelled_img, self.pet_img)
            labelled_array = sitk.GetArrayFromImage(labelled_img).transpose()
            number_of_label = stats.GetNumberOfLabels()
            volume_voxel = self.pet_spacing[0] * self.pet_spacing[1] * self.pet_spacing[2] * 10**(-3)
            for i in range(1, number_of_label + 1) :
                volume_roi = stats.GetNumberOfPixels(i) * volume_voxel
                if volume_roi < float(3.0) : 
                    x,y,z = np.where(labelled_array == i)
                    for j in range(len(x)):
                        labelled_array[x[j], y[j], z[j]] = 0


            new_binary_array = np.zeros((self.size_matrix))
            new_binary_array[np.where(labelled_array != 0)] = 1

            #new_binary_img = sitk.GetImageFromArray(new_binary_array.transpose().astype(np.uint8))
            #new_binary_img.SetOrigin(self.pet_origin)
            #new_binary_img.SetSpacing(self.pet_spacing)
            #new_binary_img.SetDirection(self.pet_direction)

            #return new_binary_img
            return new_binary_array.astype(np.uint8)

    #get labelled array/img with threshold
    def get_labelled_threshold_mask_array(self, binary_array) : 
        if len(binary_array.shape) != 3 : 
            raise Exception("Not a 3D mask, need to transform into 3D binary mask")

        else : 
            labelled_threshold_array, number_features = label(binary_array, connectivity=1, return_num = True)
            #labelled_threshold_img = sitk.ConnectedComponent(binary_img)
            return labelled_threshold_array, number_features


    def get_labelled_threshold_mask_img(self, labelled_threshold_array):
        img = sitk.GetImageFromArray(labelled_threshold_array.transpose())
        img.SetDirection(self.pet_direction)
        img.SetOrigin(self.pet_origin)
        img.SetSpacing(self.pet_spacing)
        return img

 

    #get stats results
    def label_stat_results(self, labelled_threshold_img) :
        results = {} 
        stats = sitk.LabelIntensityStatisticsImageFilter()
        stats.Execute(labelled_threshold_img, self.pet_img)
        number_of_label = stats.GetNumberOfLabels()
        results['number_of_label'] =  number_of_label
        volume = 0
        for i in range(1, number_of_label + 1) :
            subresult = {}
            subresult['max'] = stats.GetMaximum(i)
            subresult['mean'] = stats.GetMean(i)
            subresult['median'] = stats.GetMedian(i)
            subresult['variance'] = stats.GetVariance(i)
            subresult['sd'] = stats.GetStandardDeviation(i)
            subresult['number_of_pixel'] = stats.GetNumberOfPixels(i)
            volume_voxel = self.pet_spacing[0] * self.pet_spacing[1] * self.pet_spacing[2] * 10**(-3) #ml
            subresult['volume'] = stats.GetNumberOfPixels(i) * volume_voxel
            subresult['centroid'] = stats.GetCentroid(i)
            volume += stats.GetNumberOfPixels(i) * volume_voxel
            results[i] = subresult
        results['total_vol'] = volume
        return results

    #get coordonate of each roi 
    def label_coordonate(self, labelled_mask, number_of_label):
        results = {}
        for i in range(1, number_of_label + 1) :
            subliste = []
            subliste.append(np.where(labelled_mask == i))
            results[i] = subliste
            
        return results
    


from library_dicom.dicom_processor.model.csv_reader.CsvReader import CsvReader
import numpy as np 


def get_total_volume(csv_path):
    csv_objet = CsvReader(csv_path)
    total_volume = float(csv_objet.get_sum_row()[3])
    return total_volume


def keep_csv_higher_volume(liste_csv_path):
    """Return CSV to delete (not the higher volume)

    Args:
        liste_csv_path ([type]): [description]

    """
    volume = []
    for csv_file in liste_csv_path : 
        volume.append(get_total_volume(csv_file))
    index_maxi = volume.index(np.max(volume))
    if type(index_maxi) == list: 
        index_maxi = index_maxi[0]
    index = np.arange(len(volume)).tolist()
    remove = []
    for i in index:
        if i != index_maxi : 
            remove.append(liste_csv_path[i])

    return remove


    


def get_sum_rois_volume(liste):
    """sum all ROIs volume

    Args:
        liste ([list]): [List of every ROI volume]

    Returns:
        [float]: [Return the sum of ROIs volume]
    """
    volume = float(0)
    for i in range(len(liste)):
        volume += float(liste[i][3])

    return volume


def get_difference_volume(csv_path) :
    """Calcul the difference between volume and sum of every ROI volume

    Args:
        csv_path ([str]): [CSV path]

    Returns:
        [float, float, float]: [Return volume, sum of every ROIs volume and difference between the 2]
    """
    csv_objet = CsvReader(csv_path)
    total_volume = float(csv_objet.get_sum_row()[3])

    liste = csv_objet.get_rois_results()

    sum_volume = get_sum_rois_volume(liste)
    return total_volume, sum_volume, abs(sum_volume - total_volume)





from library_dicom.dicom_processor.model.Fusion import Fusion 
import SimpleITK as sitk 

class FusionMask(Fusion):
    """ReshapeResampleAlign CT/PET with MASK 
    If 'dict' or 'img'. Missing 'serie' option ! 

    Args:
        Fusion ([type]): [description]
        CT = MASK HERE (check heritance)
    """
    def __init__(self, pet, ct, target_size=None, target_spacing=None, target_direction=None, mode = 'dict'):
        super().__init__(pet, ct, target_size, target_spacing, target_direction, mode='dict')
        #self.pet_objet = pet
        #self.ct_object = mask #change name in Fusion 
        #self.target_size = target_size
        #self.target_spacing = target_spacing
        #self.target_direction = target_direction
        #self.mode = mode


    def resample(self, array = False): 
        _, mask_img = self.generate_pet_ct_img()
        target_spacing, target_direction, target_origin, target_size = self.get_feature_pet_img()
        _, _, _, mask_size = self.get_feature_ct_img()
        
        if len(mask_size) == 3 : 
            transformation = sitk.ResampleImageFilter()
            transformation.SetOutputDirection(target_direction)
            transformation.SetOutputOrigin(target_origin)
            transformation.SetOutputSpacing(target_spacing)
            transformation.SetSize(target_size)
            transformation.SetDefaultPixelValue(0.0)
            transformation.SetInterpolator(sitk.sitkNearestNeighbor)
            img = transformation.Execute(mask_img)

        
        else : #mask 4D => mask 4D
            liste = []
            for roi in range(mask_size[3]):
                extract = sitk.ExtractImageFilter()
                extract.SetSize([mask_size[0], mask_size[1], mask_size[2], 0])
                extract.SetIndex([0,0,0,roi])
                extracted_img = extract.Execute(mask_img)
                transformation = sitk.ResampleImageFilter()
                transformation.SetOutputDirection(target_direction)
                transformation.SetOutputOrigin(target_origin)
                transformation.SetOutputSpacing(target_spacing)
                transformation.SetSize(target_size)
                transformation.SetDefaultPixelValue(0.0)
                transformation.SetInterpolator(sitk.sitkNearestNeighbor)
                new_mask_img = transformation.Execute(extracted_img)
                liste.append(new_mask_img)

            img = sitk.JoinSeries(liste)


        if array==True : 
            mask_array = sitk.GetArrayFromImage(img)
            return mask_array 

        return img




import pydicom 
import numpy as np
from library_dicom.export_segmentation.tools.rtss_writer_tools import *

class StructureSetROISequence : 

    def __init__(self, mask, results):
        self.mask = mask
        self.number_of_roi = get_number_of_roi(self.mask)
        self.results = results


    def create_StructureSetROISequence(self, pixel_spacing, ReferencedFrameOfReferenceUID) :
        StructureSetROISequence = pydicom.sequence.Sequence()

        for number_roi in range(1 , self.number_of_roi +1):
            dataset = pydicom.dataset.Dataset()
            dataset.ROINumber = number_roi
            dataset.ReferencedFrameOfReferenceUID = ReferencedFrameOfReferenceUID 
            if self.results['segmentAttributes'][0][number_roi-1]["SegmentDescription"] != "None": 
                dataset.ROIName = self.results['segmentAttributes'][0][number_roi-1]["SegmentDescription"]

            else : dataset.ROIName = str(number_roi)

            dataset.ROIVolume = str(self.get_roi_volume(number_roi, pixel_spacing))
                
             
            #if self.dict_roi_data[number_roi]['ROIGenerationAlgorithm'] != '' :
            #    dataset.ROIGenerationAlgorithm = self.dict_roi_data[number_roi]['ROIGenerationAlgorithm']
            #else : dataset.ROIGenerationAlgorithm = 'UNDEFINED'
            dataset.ROIGenerationAlgorithm = self.results['segmentAttributes'][0][number_roi-1]["SegmentAlgorithmType"]
            StructureSetROISequence.append(dataset)
        return StructureSetROISequence 
        
     
    def get_roi_volume(self, number_roi, pixel_spacing):
        number_pixel = 0

        if len(self.mask.shape) == 3 :
             x = np.where(self.mask == number_roi)[0]
        elif len(self.mask.shape) == 4 : 
            if self.mask.shape[-1] != 1 : 
                x = np.where(self.mask[:,:,:,number_roi-1] != 0)[0]
            else : x = np.where(self.mask == number_roi)[0]
        number_pixel = len(x) #same as len(y) or len(z)
        volume_pixel = pixel_spacing[0] * pixel_spacing[1] * abs(pixel_spacing[2])
        volume_pixel = volume_pixel * 10**(-3) #mm3 to ml 
        roi_volume = number_pixel * volume_pixel 

        return np.round(roi_volume , 5)




from library_dicom.export_segmentation.rtstruct.RTSS_Writer import RTSS_Writer
from library_dicom.export_segmentation.dicom_seg.DICOMSEG_Writer import DICOMSEG_Writer


class ExportSegmentation_Writer : 

    def __init__(self, segmentation, serie_path, mode):
        """[summary]

        Args:
            segmentation ([array or sitk img]): [if mode = rtstruct : array, if mode = dicomseg : sitk img]
            serie_path ([str]): [Path to an imaging serie related to the segmentation ]
            mode ([str]): ['rtstruct' or 'dicomseg']
        """
        self.segmentation = segmentation
        self.serie_path = serie_path 
        self.mode = mode 
        mode_available = ['rtstruct', 'dicomseg']
        if self.mode not in mode_available : 
            raise Exception ('Format not available')
        self.is_segmentation_in_good_modality()


    def is_segmentation_in_good_modality(self):
        if self.mode == 'rtstruct' : 
            if str(type(self.segmentation)) != "<class 'numpy.ndarray'>" : 
                raise Exception ('For RTSTRUCT format, segmentation has to be an array')
        if self.mode == 'dicomseg' : 
            if str(type(self.segmentation)) != "<class 'SimpleITK.SimpleITK.Image'>" : 
                raise Exception ('For DICOM-SEG format, segmentation has to be a SITK Image')

    def generate_dicom(self, filename, directory_path):
        if self.mode == 'rtstruct' : 
            writer = RTSS_Writer(self.segmentation, self.serie_path)
            writer.save_file(filename, directory_path) 
        elif self.mode == 'dicomseg' : 
            writer = DICOMSEG_Writer(self.segmentation, self.serie_path)
            writer.save_file(filename, directory_path)



import setuptools 

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="dicom_to_cnn",                     # This is the name of the package
    version="0.43",                        # The initial release version
    author="",                     # Full name of the author
    description="Python Library to handle Input / Output conversion in Dicom <=> Convolutional Neural Network ",
    long_description=long_description,      # Long description read from the the readme file
    long_description_content_type="text/markdown",
    url = 'https://github.com/wendyrvllr/Dicom-To-CNN',   # Provide either the link to your github or to your website
    download_url = 'https://github.com/wendyrvllr/Dicom-To-CNN/archive/43.tar.gz',
    packages=setuptools.find_packages(),    # List of all python modules to be installed
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Operating System :: OS Independent",
    ],                                      # Information to filter the project on PyPi website
    python_requires='>=3.7',                # Minimum version requirement of the package
    install_requires=[]                     # Install other dependencies if any
)


import os 
import pydicom
import numpy as np 
from library_dicom.dicom_processor.model.reader.Instance import Instance
from library_dicom.dicom_processor.model.reader.Instance_RTSS import Instance_RTSS



class Instance_RTDose(Instance): 

    def __init__(self, path_dose):
        super().__init__(path_dose, load_image=True)

    def get_image_nparray(self):
        return np.transpose(self.dicomData.pixel_array, axes = (1,2,0))


    def wrap_DHVSequence_in_dict(self): 
        dictionnary = {}
        for i in range(len(self.dicomData.DVHSequence)) : 
            referenced_roi = int(self.dicomData.DVHSequence[i].DVHReferencedROISequence[0].ReferencedROINumber)
            dataset = {}
            dataset['ReferencedROINumber'] = referenced_roi
            dataset['DVHType'] = self.dicomData.DVHSequence[i].DVHType
            dataset['DoseUnits'] = self.dicomData.DVHSequence[i].DoseUnits  
            dataset['DVHDoseScaling'] = self.dicomData.DVHSequence[i].DVHDoseScaling 
            dataset['DVHVolumeUnits'] = self.dicomData.DVHSequence[i].DVHVolumeUnits 
            dataset['DVHNumberOfBins'] = self.dicomData.DVHSequence[i].DVHNumberOfBins 
            dataset['DVHMinimumDose'] = self.dicomData.DVHSequence[i].DVHMinimumDose
            dataset['DVHMaximumDose'] = self.dicomData.DVHSequence[i].DVHMaximumDose
            dataset['DVHMeanDose'] = self.dicomData.DVHSequence[i].DVHMeanDose
            dataset['DVHData'] = self.dicomData.DVHSequence[i].DVHData 
            dictionnary[referenced_roi] = dataset 

        return dictionnary
 
    def add_ROIName_from_RTSS_to_dict(self, dictionnary, rtss_file):
        instance_rtss_object = Instance_RTSS(rtss_file)
        keys = list(dictionnary.keys())
        for key in keys : 
            referenced_number = int(dictionnary[key]['ReferencedROINumber'])
            dictionnary[key]['ReferencedROIName'] = instance_rtss_object.get_ROIName_by_number(referenced_number)
        return dictionnary 


    

        


    #DHV Sequence 


    



import pydicom_seg 
import pydicom 
import numpy as np 
import json 
import os
from library_dicom.export_segmentation.tools.generate_dict import *
import SimpleITK as sitk 


class DICOMSEG_Writer:

    def __init__(self, img_mask, serie_path):
        """[summary]

        Args:
            img_mask ([sitk Image]): [sitk image of mask]
            serie_path ([str]): [Path to an imaging serie related to the segmentation ]
        """

        self.img_mask = img_mask
        self.serie_path = serie_path

    def generate_dict_json(self, directory_path):
        pred_array = sitk.GetArrayFromImage(self.img_mask)
        number_of_roi = np.max(pred_array)
        results = generate_dict(number_of_roi)
        self.results = results
        json_path = save_dict_as_json(results, directory_path)
        return json_path 



    def dicom_seg_writer(self, directory_path):
        json_path = self.generate_dict_json(directory_path)

        segmentation = self.img_mask
        dicom_file = os.listdir(self.serie_path)
        dicom_series_paths = [os.path.join(self.serie_path, x) for x in dicom_file]
        source_images = [
            pydicom.dcmread(x, stop_before_pixels=True)
            for x in dicom_series_paths]

        template = pydicom_seg.template.from_dcmqi_metainfo(json_path)

        writer = pydicom_seg.MultiClassWriter(
                template=template,
                inplane_cropping=False,  # Crop image slices to the minimum bounding box on
                                        # x and y axes. Maybe not supported by other frameworks.
                skip_empty_slices=True,  # Don't encode slices with only zeros
                skip_missing_segment=False,  # If a segment definition is missing in the
                                            # template, then raise an error instead of
                                            # skipping it.
            )

        dcm = writer.write(segmentation, source_images)
        return dcm


    def save_file(self, filename, directory_path):
        dcm = self.dicom_seg_writer(directory_path)
        dcm.save_as(os.path.join(directory_path, filename))
        return None 